import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.util.function.Consumer;

public class FormGasolineTankerConfig extends JFrame{
    private final Action<DrawingGasolineTanker> event = new Action<>();
    private DrawingGasolineTanker gasolineTanker;
    CountWheels wheels;
    private IDrawningObjectWheels _wheel=null;
    private JPanel MainPanel;
    private JPanel PreviewPanel;
    private JButton ButtonAdd;
    private JButton ButtonCancel;
    private JLabel BodyColorLabel;
    private JLabel DopColorLabel;
    private JPanel PictureBox;
    private JPanel GroupBoxConfig;
    private JSpinner SpinnerWheels;
    private JLabel CountWheelsLabel;
    private JPanel TypeWheels;
    private JLabel WheelsLabel;
    private JLabel WheelsFirstLabel;
    private JLabel WheelsSecondLabel;
    private JPanel ColorPanel;
    private JSpinner SpinnerSpeed;
    private JCheckBox CheckBoxAntenna;
    private JCheckBox checkBoxBodyKit;
    private JSpinner SpinnerWeight;
    private JPanel RedPanel;
    private JPanel OrangePanel;
    private JPanel YellowPanel;
    private JPanel CianPanel;
    private JPanel PinkPanel;
    private JPanel GreenPanel;
    private JPanel BluePanel;
    private JPanel GrayPanel;
    private JLabel BaseGasolineTankerLabel;
    private JLabel ImprovedGasolineTankerLabel;
    private JLabel SpeedLabel;
    private JLabel WeightLabel;

    public FormGasolineTankerConfig(){
        this.setTitle("Create object");
        this.setSize(900,500);
        this.setContentPane(MainPanel);
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        WheelsLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        WheelsFirstLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        WheelsFirstLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        BodyColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        DopColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        BaseGasolineTankerLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        ImprovedGasolineTankerLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        SpinnerWheels.setModel(new SpinnerNumberModel(2, 2, 4, 1));
        SpinnerWeight.setModel(new SpinnerNumberModel(100, 100, 1000, 10));
        SpinnerSpeed.setModel(new SpinnerNumberModel(10000, 10000, 20000, 100));

        var dragAdapter = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                super.mouseReleased(e);
                setCursor(new Cursor(Cursor.HAND_CURSOR));
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                super.mouseReleased(e);
                dispatchDrop((JComponent) e.getSource());
            }
        };

        RedPanel.addMouseListener(dragAdapter);
        GreenPanel.addMouseListener(dragAdapter);
        BluePanel.addMouseListener(dragAdapter);
        YellowPanel.addMouseListener(dragAdapter);
        OrangePanel.addMouseListener(dragAdapter);
        GrayPanel.addMouseListener(dragAdapter);
        CianPanel.addMouseListener(dragAdapter);
        PinkPanel.addMouseListener(dragAdapter);

        BaseGasolineTankerLabel.addMouseListener(dragAdapter);
        ImprovedGasolineTankerLabel.addMouseListener(dragAdapter);
        WheelsLabel.addMouseListener(dragAdapter);
        WheelsFirstLabel.addMouseListener(dragAdapter);
        WheelsSecondLabel.addMouseListener(dragAdapter);

        ButtonAdd.addActionListener(e -> {
            event.emit(gasolineTanker);
            dispose();
        });
        ButtonCancel.addActionListener(e -> dispose());
    }

    public void addListener(Consumer<DrawingGasolineTanker> listener) {
        event.addListener(listener);
    }

    public void dispatchDrop(JComponent droppedComponent) {
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        if (droppedComponent == null) {
            return;
        }

        if (droppedComponent instanceof JPanel panel) {
            if (BodyColorLabel.getMousePosition() != null) {
                gasolineTanker.SetBodyColor(panel.getBackground());
            }
            if (DopColorLabel.getMousePosition() != null && gasolineTanker instanceof DrawingImprovedGasolineTanker advanced) {
                advanced.SetDopColor(panel.getBackground());
            }
        }
        if (droppedComponent instanceof JLabel label && PictureBox.getMousePosition() != null) {
            int speed = (Integer) SpinnerSpeed.getValue();
            int weight = (Integer) SpinnerWeight.getValue();
            int countWheels = (Integer) SpinnerWheels.getValue();
            boolean antenna = CheckBoxAntenna.isSelected();
            boolean bodyKit = checkBoxBodyKit.isSelected();


            if (label == BaseGasolineTankerLabel) {
                gasolineTanker = new DrawingGasolineTanker(speed, weight, countWheels,Color.WHITE);
            } else if (label == ImprovedGasolineTankerLabel) {
                gasolineTanker = new DrawingImprovedGasolineTanker(speed, weight,countWheels, Color.WHITE, Color.WHITE, bodyKit, antenna);
            } else if (gasolineTanker != null && label == WheelsLabel) {
                gasolineTanker.SetWheels(new DrawingWheels(countWheels));
            } else if (gasolineTanker != null && label == WheelsFirstLabel) {
                gasolineTanker.SetWheels(new DrawingOrnamentWheelsFirst(countWheels));
            } else if (gasolineTanker != null && label == WheelsSecondLabel) {
                gasolineTanker.SetWheels(new DrawingOrnamentWheelsSecond(countWheels));
            }
        }
        repaint();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if (gasolineTanker != null) {
            g = PictureBox.getGraphics();
            gasolineTanker.SetPosition(10, 10, PictureBox.getWidth(), PictureBox.getHeight());
            gasolineTanker.DrawTransport((Graphics2D) g);
        }
    }

    private void createUIComponents() {
    }
}
