import javax.swing.*;
import java.awt.*;
import java.util.HashMap;

public class FormMapWithSetGasolineTanker extends JFrame{
    private JPanel GroupBoxTools;
    private JComboBox СomboBoxSelectorMap;
    private JButton ButtonAddGasolineTanker;
    private JButton ButtonRemoveGasolineTanker;
    private JButton ButtonShowStorage;
    private JButton ButtonShowOnMap;
    private JButton ButtonLeft;
    private JButton ButtonDown;
    private JButton ButtonRight;
    private JButton ButtonUp;
    private JPanel MainPanel;
    private JTextField TextBoxPosition;
    private JPanel PictureBox;
    private JPanel MapPanel;
    private JTextField TextFieldMap;
    private JButton CreateMapButton;
    private JButton DeleteMapButton;
    private JButton ButtonShowDeleted;
    private JList ListBoxMaps;
    private Image bufferedImage;

    private MapsCollection _mapsCollection;
    private final HashMap<String,AbstractMap> _mapsDict=new HashMap<String,AbstractMap>(){
        {
            put("Simple map",new SimpleMap());
            put("Long map",new LongMap());
        }
    };

    private MapWithSetGasolineTankerGeneric<DrawingObjectGasolineTanker,AbstractMap> _mapGasolineTankerCollectionGeneric;

    ImageIcon spriteUp =new ImageIcon((new ImageIcon("Material\\KeyUp.png")).
            getImage().getScaledInstance(10,10,Image.SCALE_SMOOTH));
    ImageIcon spriteDown =new ImageIcon((new ImageIcon("Material\\KeyDown.png")).
            getImage().getScaledInstance(10,10,Image.SCALE_SMOOTH));
    ImageIcon spriteLeft =new ImageIcon((new ImageIcon("Material\\KeyLeft.png")).
            getImage().getScaledInstance(10,10,Image.SCALE_SMOOTH));
    ImageIcon spriteRight =new ImageIcon((new ImageIcon("Material\\KeyRight.png")).
            getImage().getScaledInstance(10,10,Image.SCALE_SMOOTH));
    public FormMapWithSetGasolineTanker(){
        setTitle("Gasoline tanker");
        setContentPane(MainPanel);
        setResizable(false);
        setSize(1000,685);
        ShowWindow();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);
        _mapsCollection=new MapsCollection(getWidth(),getHeight());
        СomboBoxSelectorMap.removeAllItems();
        for(String elem:_mapsDict.keySet()){
            СomboBoxSelectorMap.addItem(elem);
        }
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (bufferedImage != null) {
            PictureBox.paintComponents(bufferedImage.getGraphics());
            PictureBox.getGraphics().drawImage(bufferedImage, 0, 0, null);
        }
    }

    private void ReloadMaps()
    {
        int index = ListBoxMaps.getSelectedIndex();
        ListBoxMaps.setListData(_mapsCollection.Keys().toArray(new String[0]));

        if (ListBoxMaps.getModel().getSize() > 0 && (index == -1 || index >= ListBoxMaps.getModel().getSize()))
        {
            ListBoxMaps.setSelectedIndex(0);
        }
        else if (ListBoxMaps.getModel().getSize() > 0 && index > -1 && index < ListBoxMaps.getModel().getSize())
        {
            ListBoxMaps.setSelectedIndex(index);
        }
        repaint();
    }

    private void ShowWindow(){

        ListBoxMaps.addListSelectionListener(e -> {
            if(ListBoxMaps.getSelectedIndex()==-1)
                return;
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        DeleteMapButton.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() == -1)
            {
                return;
            }
            if(JOptionPane.showConfirmDialog(this,"Delete map "+ListBoxMaps.getSelectedValue().toString()+"?",
                    "Удаление",JOptionPane.YES_NO_OPTION)==0)
            {
                _mapsCollection.DelMap(ListBoxMaps.getSelectedValue().toString());
                ReloadMaps();
            }
        });

        CreateMapButton.addActionListener(e -> {
            if (СomboBoxSelectorMap.getSelectedIndex() == -1 || TextFieldMap.getText()==null || TextFieldMap.getText().equals(""))
            {
                JOptionPane.showMessageDialog(this,"\n" + "Not all data is complete","Error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (!_mapsDict.containsKey(СomboBoxSelectorMap.getSelectedItem()))
            {
                JOptionPane.showMessageDialog(this,"No such card","Error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            _mapsCollection.AddMap(TextFieldMap.getText(), _mapsDict.get(СomboBoxSelectorMap.getSelectedItem().toString()));
            ReloadMaps();
        });

        ButtonShowOnMap.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowOnMap();
            repaint();
        });

        ButtonShowStorage.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        ButtonAddGasolineTanker.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            FormGasolineTankerConfig dialog=new FormGasolineTankerConfig();
            dialog.addListener(obj -> {
                if (obj!=null) {
                    DrawingObjectGasolineTanker gasolineTanker = new DrawingObjectGasolineTanker(obj);

                    if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).plus(gasolineTanker) >= 0) {
                        JOptionPane.showMessageDialog(this, "Object added", "Success", JOptionPane.INFORMATION_MESSAGE);
                        bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                        repaint();
                    } else {
                        JOptionPane.showMessageDialog(this, "\n" + "Failed to add object", "Error", JOptionPane.INFORMATION_MESSAGE);
                    }
                }
            });
            dialog.setVisible(true);
        });

        ButtonRemoveGasolineTanker.addActionListener(e -> {
            String txt=TextBoxPosition.getText();
            if (txt==null||ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            int result = JOptionPane.showConfirmDialog(
                    this,
                    "Delete object?",
                    "Removal",
                    JOptionPane.YES_NO_CANCEL_OPTION);
            if (result!=0)
            {
                return;
            }
            int pos = Integer.parseInt(txt);
            if(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).minus(pos)!=null){
                JOptionPane.showMessageDialog(this, "Object removed", "Success", JOptionPane.INFORMATION_MESSAGE);
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                repaint();
            }
            else
            {
                JOptionPane.showMessageDialog(this,"Failed to delete object","\n" + "Error",JOptionPane.INFORMATION_MESSAGE);
            }
        });

        ButtonShowDeleted.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex()==-1)
                return;
            DrawingObjectGasolineTanker gasolineTanker=(DrawingObjectGasolineTanker)_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).GetGasolineTankerInDeleted();
            if(gasolineTanker!=null){
                FormGasolineTanker dialog=new FormGasolineTanker();
                dialog.SetDrawingGasolineTanker(gasolineTanker.GetGasolineTanker());
                dialog.setSize(1200,700);
                dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
                dialog.setVisible(true);
            }
        });
        ButtonUp.setIcon(spriteUp);
        ButtonUp.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Up);
                repaint();
            }
        });
        ButtonDown.setIcon(spriteDown);
        ButtonDown.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Down);
                repaint();
            }
        });
        ButtonRight.setIcon(spriteRight);
        ButtonRight.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Right);
                repaint();
            }
        });
        ButtonLeft.setIcon(spriteLeft);
        ButtonLeft.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Left);
                repaint();
            }
        });
    }
}
